/*
*************************************************************************************
* Copyright 2011 Normation SAS
*************************************************************************************
*
* This file is part of Rudder.
* 
* Rudder is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* In accordance with the terms of section 7 (7. Additional Terms.) of
* the GNU General Public License version 3, the copyright holders add
* the following Additional permissions:
* Notwithstanding to the terms of section 5 (5. Conveying Modified Source
* Versions) and 6 (6. Conveying Non-Source Forms.) of the GNU General
* Public License version 3, when you create a Related Module, this
* Related Module is not considered as a part of the work and may be
* distributed under the license agreement of your choice.
* A "Related Module" means a set of sources files including their
* documentation that, without modification of the Source Code, enables
* supplementary functions or services in addition to those offered by
* the Software.
* 
* Rudder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Rudder.  If not, see <http://www.gnu.org/licenses/>.

*
*************************************************************************************
*/

package com.normation.rudder.services.policies

import net.liftweb.common._
import org.apache.commons.io.FilenameUtils
import org.apache.commons.io.FileUtils
import java.io.File
import java.io.IOException
import com.normation.rudder.repository.LicenseRepository
import com.normation.cfclerk.domain.PromisesFinalMoveInfo
import com.normation.inventory.domain.NodeId
import com.normation.rudder.services.policies.nodeconfig.NodeConfiguration
import com.normation.rudder.domain.reports.NodeConfigId

trait TemplateWriter extends Loggable {
  def licenseRepository : LicenseRepository

  def getSharesFolder() : String
  
  def reloadCFEnginePromises() : Unit

  /**
   * Write the promises of all the nodes
   * @param updateBatch : the container for the server to be updated
   */
  def writePromisesForMachines(configToWrite: Set[NodeId], rootNodeId: NodeId, allNodeConfigs:Map[NodeId, NodeConfiguration], versions: Map[NodeId, NodeConfigId]) : Box[Seq[PromisesFinalMoveInfo]]

  def writeLicense(nodeConfiguration : NodeConfiguration, newMachineFolder:String) : Unit = {
    logger.debug("Writing licence for nodeConfiguration  " + nodeConfiguration.nodeInfo.id);
    nodeConfiguration.nodeInfo.isPolicyServer match {
      case true =>  copyLicenseFile(nodeConfiguration.nodeInfo.id, newMachineFolder)

      case false => copyLicenseFile(nodeConfiguration.nodeInfo.policyServerId, newMachineFolder)
    }
  }

  private def copyLicenseFile(nodeConfigurationid: NodeId, newMachineFolder:String) : Unit = {
    licenseRepository.findLicense(nodeConfigurationid) match {
      case None =>
        // we are in the "free case", just log-debug it (as we already informed the user that there is no license)
        logger.info(s"Not copying missing license file into '${newMachineFolder}' for node '${nodeConfigurationid.value}'.")

      case Some(license) =>
        val licenseFile = new File(license.file)
        if (licenseFile.exists) {
          val destFile = FilenameUtils.normalize(newMachineFolder + "/license.dat")
          FileUtils.copyFile(licenseFile, new File(destFile) )
        } else {
          logger.error("Could not find the license file %s for server %s".format(license.file, nodeConfigurationid))
          throw new Exception("Could not find license file " +license.file)
        }
    }
  }


  /**
   * Create a sym link from a file to the folder of a nodeConfiguration
   * The goal is when we want to deploy a file on several nodeConfigurations, we simply create a link
   * on the server, that will get followed
   */
  private def createSymLink(fileName :  String, newMachineFolder:String) : Unit = {
    val source = FilenameUtils.normalize(getSharesFolder() + "/" + fileName)
    val dest = FilenameUtils.normalize(newMachineFolder + "/shares/" + fileName)
    FileUtils.forceMkdir( new File(FilenameUtils.normalize(newMachineFolder + "/shares/")))

    val command = Array[String]("ln", "-fs", "source", "dest")

    //val exec ="ln"+" -fs "+source+" " +dest

    val process = Runtime.getRuntime().exec(command);

    if (process.waitFor() != 0) {
      logger.error("Couldn't successfully create the link exec")
      throw new IOException("Couldn't link file " + source)
    }

    logger.info("Making a link from " + source + " to " + newMachineFolder  + "/shares/" +fileName );

  }
}
